<?php
/*
Plugin Name: Birthdaze
Plugin URI: http://familypress.net/birthdaze/
Description: A very simple birthday plugin for WordPress, with a birthday manager, template function, and a widget. Last tested and working with WordPress 2.7.1.
Version: 0.3
Author: Isaac Wedin
Author URI: http://familypress.net/
*/

/* INSTALLATION: It should only be necessary to activate it - that should install or update the table.
*/

/* Copyright 2009  Isaac Wedin (email : isaac@familypress.net)
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Tell the localization loader the name of the plugin so it can load a translation if one is needed and installed.
load_plugin_textdomain('birthdaze');

// Fill the options array with stored options data if there's any there.
$birthdaze_options = array();

if (get_option('birthdaze_options')) {
	$birthdaze_options = get_option('birthdaze_options');
}

// We'll set a default for each option that doesn't have a saved value.

// 
if (empty($birthdaze_options['capability'])) {
	$birthdaze_options['capability'] = 'edit_posts';
}

// only those with the 'manage_options' capability will see the Options
function birthdaze_add_options_page() {
	add_options_page('Birthdaze', 'Birthdaze', 'manage_options', basename(__FILE__), 'birthdaze_options_subpanel');
}

// generates the Birthdaze Options subpanel
function birthdaze_options_subpanel() {
	global $birthdaze_options;
	if (isset($_POST['birthdaze_options_update'])) { // update the options and refresh the options array with the updated options
		$birthdaze_updated_options = array();
		$birthdaze_updated_options = $_POST;
		add_option('birthdaze_options');
		update_option('birthdaze_options', $birthdaze_updated_options);
		$birthdaze_options = get_option('birthdaze_options');
		echo '<div class="updated">' . __('Birthdaze options updated.','birthdaze') . '</div>';
	}
	echo '
	<div class="wrap">
	<h2>' . __('Birthdaze Options','birthdaze') . '</h2>
	<form name="birthdaze_options" method="post">
	<input type="hidden" name="birthdaze_options_update" value="update" />
	<fieldset class="options">
	<table width="100%" cellspacing="2" cellpadding="5" class="editform">
		<tr>
			<th scope="row">' . __('Capability required to use Birthdaze:','birthdaze') . '</th>
			<td><input type="text" size="20" name="capability" id="capability" value="' . $birthdaze_options['capability'] . '" /><br />
		' . __('You probably want to use <code>edit_posts</code>, but you could also use <code>upload_files</code>, <code>publish_posts</code> or any other capability you want.','birthdaze') . '</td>
		</tr>
	</table> 
	</fieldset>

	<p class="submit"><input type="submit" name="Submit" value="' . __('Update Options &raquo;','birthdaze') . '" /></p>
</form> 
</div>
';
}

add_action('admin_menu', 'birthdaze_add_options_page');

// makes the Birthdaze Management page
function birthdaze_management() {
	global $birthdaze_options, $wpdb, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	if (isset($_POST['birthdaze_update'])) {
		$dels = 0;
		$updates = 0;
		foreach ((array)$_POST as $key=>$val) {
			if (is_array($val)) {
				if (($val['delete'] == '1')) {
					if (birthdaze_delete($val['id'])) {
						$dels++;
					}
				} elseif (!empty($val['fname'])) {
					$update_array = array();
					$update_array['id'] = $val['id'];
					$update_array['fname'] = $val['fname'];
					$update_array['lname'] = $val['lname'];
					$update_array['bmonth'] = $val['bmonth'];
					$update_array['bday'] = $val['bday'];
					if (birthdaze_update($update_array)) {
						$updates++;
					}
				}
			}
		}
		echo "<div class='updated'>";
		if ($dels > 0) {
			echo $dels . __(' records deleted. ','birthdaze');
		}
		if ($updates > 0) {
			echo $updates . __(' records updated.','birthdaze');
		}
		echo "</div>\n";
	}
	echo '
	<div class="wrap">
	<h2>' . __('Birthday Manager','birthdaze') . '</h2>
	<p>' . __('Add a birthday or edit the existing birthdays. Enter at least a first name and month. Only the first letter of the last name will be displayed on the site.','birthdaze') . '</p>
	<form name="birthdaze" method="post">
	<p class="submit"><input type="submit" name="Submit" value="' . __('Update','birthdaze') . ' &raquo;" /></p>
	<input type="hidden" name="birthdaze_update" value="update" />
	<table>
	<tr>
		<th scope="col">' . __('First','birthdaze') . '</th>
		<th scope="col">' . __('Last','birthdaze') . '</th>
		<th scope="col">' . __('Month','birthdaze') . '</th>
		<th scope="col">' . __('Day','birthdaze') . '</th>
		<th scope="col">' . __('Delete','birthdaze') . '</th>
	</tr>
	<tr>
		<td><input type="text" name="row1[fname]" value="" size="5" /></td>
		<td><input type="text" name="row1[lname]" value="" size="5" /></td>
		<td><input type="text" name="row1[bmonth]" value="" size="2" /></td>
		<td><input type="text" name="row1[bday]" value="" size="2" /></td>
		<td></td>
	</tr>
	';
	if ($contacts = $wpdb->get_results("SELECT * FROM $table_name ORDER BY lname,fname",ARRAY_A)) {
		$i = 2;
		foreach ((array)$contacts as $contact) {
			echo '<tr>
		<td><input type="hidden" name="row' . $i . '[id]" value="' . $contact['id'] . '" /><input type="text" name="row' . $i . '[fname]" value="' . $contact['fname'] . '" size="5" /></td>
		<td><input type="text" name="row' . $i . '[lname]" value="' . $contact['lname'] . '" size="5" /></td>
		<td><input type="text" name="row' . $i . '[bmonth]" value="' . $contact['bmonth'] . '" size="2" /></td>
		<td><input type="text" name="row' . $i . '[bday]" value="' . $contact['bday'] . '" size="2" /></td>
		<td><input type="checkbox" name="row' . $i . '[delete]" value="1" /></td>
	</tr>';
			$i++;
		}
	}
	echo '
		</table>
		<p class="submit"><input type="submit" name="Submit" value="' . __('Update','birthdaze') . ' &raquo;" /></p>
		</form>
	</div>
	';
}

function birthdaze_add_management_page() {
	global $birthdaze_options;
	add_management_page('Birthdaze', 'Birthdaze', $birthdaze_options['capability'], basename(__FILE__), 'birthdaze_management');
}

add_action('admin_menu', 'birthdaze_add_management_page');

// install a table in the db for birthdaze data
function birthdaze_table_install() {
	global $table_prefix, $wpdb;
	$table_name = $table_prefix . "birthdaze";
	$sql = "CREATE TABLE ".$table_name." (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		fname varchar(55) NULL,
		lname varchar(55) NULL,
		bmonth tinyint(2) NULL,
		bday tinyint(2) NULL,
		UNIQUE KEY id (id)
	);";
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
	dbDelta($sql);
}

// run the table installer when the plugin is activated
if (isset($_GET['activate']) && $_GET['activate'] == 'true') {
	add_action('init', 'birthdaze_table_install');
}

function birthdaze_update($update_array) {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	foreach ($update_array as $key=>$val) {
		$update_array[$key] = $wpdb->escape($val);
	}
	if ($wpdb->get_var("SELECT id FROM $table_name WHERE binary id = '" . $update_array['id'] . "'")) {
		return $wpdb->query("UPDATE $table_name SET 
			fname = '" . $update_array['fname'] . "', 
			lname = '" . $update_array['lname'] . "', 
			bmonth = '" . $update_array['bmonth'] . "',
			bday = '" . $update_array['bday'] . "'
			WHERE binary id = '" . $update_array['id'] . "'");
	} else {
		return $wpdb->query("INSERT INTO $table_name SET 
	fname = '" . $update_array['fname'] . "',
	lname = '" . $update_array['lname'] . "',
	bmonth = '" . $update_array['bmonth'] . "',
	bday = '" . $update_array['bday'] . "'");
	}
}

function birthdaze_delete($id) {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	$id = $wpdb->escape($id);
	if ($wpdb->query("DELETE FROM $table_name WHERE binary id = '$id'")) {
		return TRUE;
	} else {
		return FALSE;
	}
}

function birthdaze_thismonth($before='<li>',$after='</li>',$none='None found.') {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	$today = getdate();
	if ($bdays = $wpdb->get_results("SELECT fname,lname,bmonth,bday FROM $table_name WHERE bmonth = MONTH(CURDATE()) ORDER BY bday",ARRAY_A)) {
		foreach ((array)$bdays as $bday) {
			$bdaytime = strtotime($bday['bday'] . ' ' . $today['month'] . ' ' . $today['year']);
			$suffix = date('S',$bdaytime);
			if (!empty($bday['lname'])) {
				$linitial = " " . substr($bday['lname'],0,1);
			}
			echo $before . $bday['fname'] . $linitial . ": " . $bday['bday'] . $suffix . $after . "\n";
		}
	} else {
			echo $before . $none . $after . "\n";
	}
}

function birthdaze_allofem($before='<li>',$after='</li>') {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	$today = getdate();
	if ($bdays = $wpdb->get_results("SELECT fname,lname,bmonth,bday FROM $table_name ORDER BY bmonth,bday",ARRAY_A)) {
		foreach ((array)$bdays as $bday) {
			if ($bday['bday'] < 10) {
				$day = '0' . $bday['bday'];
			} else {
				$day = $bday['bday'];
			}
			if ($bday['bmonth'] < 10) {
				$month = '0' . $bday['bmonth'];
			} else {
				$month = $bday['bmonth'];
			}
			$bdaytime = strtotime($today['year'] . $month . $day);
			$bday_text = date('F',$bdaytime) . ' ' . $bday['bday'] . date('S',$bdaytime);
			if (!empty($bday['lname'])) {
				$linitial = " " . substr($bday['lname'],0,1);
			}
			echo $before . $bday['fname'] . $linitial . ": " . $bday_text . $after . "\n";
		}
	}
}

function widget_birthwidg_init() {
	if ( !function_exists('register_sidebar_widget') || !function_exists('birthdaze_thismonth'))
		return;
	function widget_birthwidg($args) {
		extract($args);
		$options = get_option('widget_birthwidg');
		$title = $options['title'];
		echo $before_widget . $before_title . $title . $after_title . "\n<ul>\n";
		birthdaze_thismonth($before='<li>',$after='</li>',$none='None found.');
		echo "\n</ul>\n" . $after_widget;
   }
	function widget_birthwidg_control() {
		$options = get_option('widget_birthwidg');
		if ( !is_array($options) )
			$options = array('title'=>'', 'widgets');
		if ( $_POST['birthwidg-submit'] ) {
			$options['title'] = strip_tags(stripslashes($_POST['birthwidg-title']));
			update_option('widget_birthwidg', $options);
		}
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		echo '<p style="text-align:right;"><label for="birthwidg-title">' . __('Title:') . ' <input style="width: 100px;" id="birthwidg-title" name="birthwidg-title" type="text" value="'.$title.'" /></label></p>';
		echo '<input type="hidden" id="birthwidg-submit" name="birthwidg-submit" value="1" />';
	}
	register_sidebar_widget(array('Birthdays', 'widgets'), 'widget_birthwidg');
	register_widget_control(array('Birthdays', 'widgets'), 'widget_birthwidg_control');
}
add_action('widgets_init', 'widget_birthwidg_init');

?>
